From df953f4769be7d905da5acce9730c54e7163afce Mon Sep 17 00:00:00 2001
From: Eyal Itkin <eyalit@checkpoint.com>
Date: Mon, 30 Mar 2020 21:12:02 +0300
Subject: [PATCH] Resolve bug when using unusual MALLOC_ALIGNMENT

Safe-Linking alignment checks should be done on the user's buffer and not
the mchunkptr. The new check adds support for cases in which:
MALLOC_ALIGNMENT != 2*(sizeof(size_t))

The default case for both 32 bits and 64 bits was already supported, and
this patch adds support for the described irregular case.
---
 libc/stdlib/malloc-standard/malloc.h | 7 +++++--
 1 file changed, 5 insertions(+), 2 deletions(-)

diff --git a/libc/stdlib/malloc-standard/malloc.h b/libc/stdlib/malloc-standard/malloc.h
index 30a696e5a..2d6503ed0 100644
--- a/libc/stdlib/malloc-standard/malloc.h
+++ b/libc/stdlib/malloc-standard/malloc.h
@@ -848,8 +848,11 @@ typedef struct malloc_chunk* mfastbinptr;
 */
 #define PROTECT_PTR(pos, ptr)     ((mchunkptr)((((size_t)pos) >> PAGE_SHIFT) ^ ((size_t)ptr)))
 #define REVEAL_PTR(pos, ptr)      PROTECT_PTR(pos, ptr)
-#define CHECK_PTR(P)    \
-  if (!aligned_OK(P))   \
+#define PTR_FOR_ALIGNMENT_CHECK(P) \
+    (MALLOC_ALIGNMENT == 2*(sizeof(size_t)) ? (P) : chunk2mem(P))
+
+#define CHECK_PTR(P)                            \
+  if (!aligned_OK(PTR_FOR_ALIGNMENT_CHECK(P)))  \
       abort();
 
 /*
-- 
2.17.1

